﻿using Microsoft.Win32;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;

namespace MojeHra
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {

        Game game;
        private bool IsSelected = false;
        private Move currentMove;
        public MainWindow()
        {
            InitializeComponent();
        }

      

        private void EndGameMenu_Click(object sender, RoutedEventArgs e)
        {
            MessageBoxResult res = MessageBox.Show("Opravdu chcete ukončit hru?", "Dotaz", MessageBoxButton.YesNo);
            if (res == MessageBoxResult.Yes)
            {
                Application.Current.Shutdown();
            }
        }

        private void LoadGameMenu_Click(object sender, RoutedEventArgs e)
        {
            OpenFileDialog dialog = new OpenFileDialog();
            dialog.InitialDirectory = @"U:\";
            dialog.Filter = "Save game File (.sav)|*.sav|Test|*.test| Všechny soubory| *.*";
            if (dialog.ShowDialog()==true)
            {
                game.LoadGame(dialog.FileName);
            }
        }

        private void NewGameMenu_Click(object sender, RoutedEventArgs e)
        {
            NewGame dialog = new NewGame();
            dialog.Owner = this;
            dialog.ShowDialog();
            if (dialog.IsCreated)
            {
                this.game = new Game(dialog.GetPlayer1(), dialog.GetPlayer2());
                showBoard();
            }
        }

        public void showBoard()
        {
            BoardCanvas.Children.Clear();
            int fieldWidth = GetFieldWidth();
            int fieldHeight = GetFieldHeight();
            List<int> coords = GetFieldCoords(fieldWidth, fieldHeight);
            DrawBoard(coords, fieldWidth, fieldHeight);
            DrawFigures(coords, fieldWidth, fieldHeight);
        }
        private void DrawFigures(List<int> coords, int fWidth, int fHeight)
        {
            for (int i=0; i<8; i++)
            {
                for (int j=0; j<8; j++)
                {
                    int[] phyCoords = MapFieldPhyCoord(coords, i, j);
                    DrawFigure(game.board[i][j], phyCoords[0], phyCoords[1], fWidth, fHeight);

                }
            }
        }

        private int[] MapFieldPhyCoord(List<int> phyCoords, int i, int j)
        {
            int[] result = new int[2];
            int x = (i * 8 + j) * 2;
            int y = x + 1;
            result[0] = phyCoords[x];
            result[1] = phyCoords[y];
            return result;
        }

        private void DrawFigure(FigureType typ, int x, int y, int width, int height)
        {
            if (typ!=FigureType.None) { 
            SolidColorBrush border = new SolidColorBrush(Colors.Black);
            SolidColorBrush c = new SolidColorBrush(Colors.Black);
            if (typ<0)
            {
                c = new SolidColorBrush(Colors.White);
            }
            DrawEllipse(x, y, width, height, c, border);
            if (Math.Abs((int)typ) >1)
            {
                DrawKing(x, y, width, height, new SolidColorBrush(Colors.Gray), border);
            }
            }
        }

        private void DrawKing(int x, int y, int fWidth, int fHeight, SolidColorBrush brush, SolidColorBrush stroke)
        {
            DrawEllipse(x + fWidth / 4, y + fHeight / 4, fWidth / 2, fHeight / 2, brush, stroke) ;
        }

        private void DrawEllipse(int x, int y, int width, int height, SolidColorBrush fiil, SolidColorBrush stroke)
        {
            Ellipse el = new Ellipse();
            el.Fill = fiil;
            el.Stroke = stroke;
            el.Width = width;
            el.Height = height;
            BoardCanvas.Children.Add(el);
            Canvas.SetLeft(el, x);
            Canvas.SetBottom(el, y);
        }




        private void DrawBoard(List<int> coords, int fWidth, int fHeight)
        {
            SolidColorBrush black = new SolidColorBrush(Colors.Gray);
            SolidColorBrush white = new SolidColorBrush(Colors.White);
            int j = 0;
            for (int i=0; i<coords.Count; i=i+2) {
                SolidColorBrush brush;
                if ((i%4)/2 == j%2)
                {
                    brush = black;
                } else
                {
                    brush = white;
                }
                DrawField(coords[i], coords[i+1], fWidth, fHeight, brush);
                if (i%16 == 14)
                {
                    j++;
                }

            }
        }

        private void DrawField(int x, int y, int width, int height, SolidColorBrush color)
        {
            Rectangle r = new Rectangle();
            r.Width = width;
            r.Height = height;
            r.Fill = color;
            BoardCanvas.Children.Add(r);
            Canvas.SetLeft(r, x);
            Canvas.SetBottom(r, y);
        }

        private int GetFieldWidth()
        {
            return (int)(BoardCanvas.Width / 8);
        }
        private int GetFieldHeight()
        {
            return (int)(BoardCanvas.Height / 8);
        }

        private List<int> GetFieldCoords(int fWidth, int fHeight)
        {
            List<int> result = new List<int>();
            for (int i = 0; i < 8; i++) {
                for (int j=0; j<8; j++)
                {
                    result.Add(j * fWidth);
                    result.Add(i * fHeight);
                }
            }
            return result;
        }

        private void BoardCanvas_MouseDown(object sender, MouseButtonEventArgs e)
        {
            int clickX = (int)e.GetPosition(BoardCanvas).X;
            int clicky = (int)e.GetPosition(BoardCanvas).Y;

            int[] boardCoords = MapClickToBoard(clickX, clicky);
            //MessageBox.Show($"Realne souradnice: {clickX}, {clicky}. Prepocitane logicke: {boardCoords[0]},{boardCoords[1]}");
            if (!IsSelected)
            {
                List<Move> possibleMoves = game.GetPossibleMoves(boardCoords[0], boardCoords[1]);
                if (possibleMoves.Any())
                {
                    DisplayPossibleMoves(possibleMoves);
                    currentMove = new Move(boardCoords[0], boardCoords[1], -1, -1);
                    IsSelected = true;
                } else
                {
                    MessageBox.Show("Figurkou nelze hybat");
                }
            } else
            {
                currentMove.x2 = boardCoords[0];
                currentMove.y2 = boardCoords[1];
                game.MakeMove(currentMove);
                IsSelected = false;
                showBoard();
            }


        }

        private void DisplayPossibleMoves(List<Move> moves)
        {
            List<int> coords = GetFieldCoords(GetFieldWidth(), GetFieldHeight());
            SolidColorBrush col = new SolidColorBrush(Color.FromArgb(155, 0, 255, 0));
            foreach (Move m in moves)
            {
                int[] rectCoor = MapFieldPhyCoord(coords, m.x2, m.y2);
                DrawField(rectCoor[0], rectCoor[1], GetFieldWidth(), GetFieldHeight(), col);
            }
        }

        private int[] MapClickToBoard(int clickX, int clickY)
        {
            clickY = (int)BoardCanvas.Height - clickY;
            List<int> coords = GetFieldCoords(GetFieldWidth(), GetFieldHeight());
            int coordI = 7;
            int coordJ = 7;
            for (int i = 0; i < coords.Count; i += 16)
            {
                if (clickY < coords[i + 1])
                {
                    coordI = (i / 16) - 1;
                    break;
                }
            }
            for (int i = 0; i < 8; i++)
            {
                if (clickX < coords[i * 2])
                {
                    coordJ = (i % 8) - 1;
                    break;
                }
            }
            return new int[] { coordI, coordJ };

        }
    }
}
